
package Archivista::BL::Archive;
use strict;
use Archivista::Config;
use Archivista::BL::Document;
use Archivista::BL::User;
use Archivista::BL::Parameter;
use Archivista::BL::Languages;
use Archivista::BL::Application;
use Archivista::DL::DB;
use Archivista::DL::Archive;
use Archivista::Util::Collection;
use Archivista::Util::IO;
use Archivista::Util::Session;
use Archivista::Util::FS;
use Archivista::Util::Jobs;

sub _init {
  my ($self,$access,$host,$uid,$pwd,$langCode,$database) = @_;
  if (length($access) == 32) {
    # PLEASE NOTE: we can't use database names of 32 bytes length!
    # We have a session id as access variable
    my $session = Archivista::Util::Session->check($access);
    # Check if the session exists
    if (defined $session) {
    	$session->read;
     	$host = $session->host;
     	$database = $session->db;
     	$uid = $session->user;
     	$pwd = $session->password;
			$langCode = $session->language;
      $self->{'session'} = $session;
	    $self->{'archive_name'} = $database;
    }
  } elsif (defined $access) {
    # In this case we open a new session
    $database = $access;
    $self->{'session'} = Archivista::Util::Session->open(
			$database,$host,$uid,$pwd,$langCode);
    $self->{'archive_name'} = $database;
  }
  $self->{'db'} = Archivista::DL::DB->new($database,$host,$uid,$pwd);
  $self->{'document_collection'} = Archivista::Util::Collection->new;
  $self->{'user_collection'} = Archivista::Util::Collection->new;
  $self->{'parameter_collection'} = Archivista::Util::Collection->new;
  $self->{'application'} = Archivista::BL::Application->new($self->{'db'});
  $self->{'lang_code'} = $langCode;
  $self->_exception();
}

sub _create {
	my ($self,$archiveName,$cpDataFromArchive,$host,$uid,$pwd) = @_;
	Archivista::DL::DB->create($archiveName,$cpDataFromArchive,$host,$uid,$pwd);
	$self->_exception();
}

sub _configure {
	my ($self,$archiveName,$cpDataFromArchive) = @_;
	my $config = Archivista::Config->new;
	if (defined $cpDataFromArchive) {
		# 2-dim array[n][3]
		my $pausers = $self->users;
		$self->grant($pausers);
	} else {
		# Create SYSOP user
		$self->user("localhost","SYSOP","",255);
		Archivista::BL::Parameter->load($self,$archiveName);
		Archivista::Util::FS->createImageFoldersForArchive($archiveName);
	}
	$self->_exception();
}

sub _documentCollection {
	my ($self,$documentId,$document) = @_;
	if (defined $document) {
		$self->collection("document")->element($documentId,$document);	
	} else {
		return $self->collection("document")->element($documentId);
	}
}

sub _userCollection {
	my ($self,$userId,$user) = @_;
	if (defined $user) {
		$self->collection("user")->element($userId,$user);
	} else {
		return $self->collection("user")->element($userId);
	}
}

sub _parameterCollection {
	my ($self,$parameterId,$parameter) = @_;
	if (defined $parameter) {
		$self->collection("parameter")->element($parameterId,$parameter);
	} else {
		return $self->collection("parameter")->element($parameterId);
	}
}

sub _exception {
	my ($self) = @_;
	if (length($@) > 0) {
		my $exception = $@;
		my $config = Archivista::Config->new;
		my $io = Archivista::Util::IO->new;
		$io->append($config->get("LOG_FILE"),$exception);
		undef $@;
    #delete $self->{'db'};
	}
}

sub load {
	my ($cls,$access,$host,$uid,$pwd,$lang) = @_;
	my $self = {};
	bless $self, $cls;
	$self->_init($access,$host,$uid,$pwd,$lang);
	return $self;
}

sub create {
	my ($cls,$archiveName,$cpDataFromArchive,$host,$uid,$pwd,$sid) = @_;
	my $self = {};
	bless $self, $cls;
	$self->_create($archiveName,$cpDataFromArchive,$host,$uid,$pwd);
	$self->_init($archiveName,$host,$uid,$pwd,$sid);
	$self->_configure($archiveName,$cpDataFromArchive);
	return $self;
}

sub drop {
	my ($self,$archive) = @_;
	# Revoke privileges of all users
	# $self->revoke($self->users);
	# Drop the database
  $self->db->drop($archive);
	# Unlink the image folders of the archive
	Archivista::Util::FS->unlinkImageFoldersForArchive($archive);
  $self->_exception();
}

sub alter {
	my ($self,$palter) = @_;
	Archivista::DL::Archive->alter($self->db,$palter);
	$self->_exception();
}

sub userDefinedAttributes {
	my ($self,$retDS) = @_;
	$retDS = "ARRAY" if (! defined $retDS);
	if (uc($retDS) eq "ARRAY") {
		return $self->db->userDefinedAttributes("ARRAY");
	} elsif (uc($retDS) eq "HASH") {
		return $self->db->userDefinedAttributes("HASH");
	}
}

sub describeAttribute {
	my ($self,$attribute) = @_;
	# Retrieve informations about an attribute
	# Name, Type (varchar,int,...), Length (1,11,..), Position after attribute
	my ($attributeType,$attributeLength,%attribute); 
	my $after = "Seiten";
	my $oudAttribute = "Seiten";
	my $paudAttributes = $self->db->userDefinedAttributes("ARRAY");
	my $phudAttributes = $self->db->userDefinedAttributes("HASH");
	# Get the attribute befor $attribute
	foreach my $udAttribute (@$paudAttributes) {
		$after = $oudAttribute if ($udAttribute eq $attribute);
		$oudAttribute = $udAttribute;
	}
	if ($$phudAttributes{$attribute} =~ /(.*?)(\()(.*)(\))/) {
  	$attributeType = $1;
		$attributeLength = $3 if ($1 eq "varchar" || $1 eq "int");
	} else {
 		$attributeType = $$phudAttributes{$attribute};
	}
	$attribute{$attribute}{'attribute_name'} = $attribute;
  $attribute{$attribute}{'attribute_type'} = $attributeType;
  $attribute{$attribute}{'attribute_length'} = $attributeLength;
  $attribute{$attribute}{'attribute_after'} = $after;
  return \%attribute;
}

sub disconnect {
	my ($self) = @_;
	$self->db->dbh->disconnect if defined $self->db->dbh;	
	$self->db->sudbh->disconnect if defined $self->db->sudbh;
  foreach my $key (keys %{$self}) {
    delete $self->{$key};	
  }
  undef $self;
}

sub curDocId {
	my ($self,$documentId) = @_;
	if (defined $documentId) {
		$self->{'cur_doc_id'} = $documentId;
	} else {
		return $self->{'cur_doc_id'};
	}
}

sub curUserId {
	my ($self,$userId) = @_;
	if (defined $userId) {
		$self->{'cur_user_id'} = $userId;
	} else {
		return $self->{'cur_user_id'};
	}
}

sub curParameterId {
  my ($self,$parameterId) = @_;
  if (defined $parameterId) {
    $self->{'cur_parameter_id'} = $parameterId;
  } else {
    return $self->{'cur_parameter_id'};
  }
}

sub name {
  my ($self) = @_;
  return $self->{'archive_name'};
}

sub collection {
  my ($self,$collectionType) = @_;
  return $self->{$collectionType.'_collection'};
}

sub db {
  my ($self) = @_;
  return $self->{'db'};
}

sub document {
  my ($self,$documentId) = @_;
  my $document;
  # Check if setter or getter method
  if (defined $documentId) {
    # Setter: load the requested document
    # and add the object to the collection
    $document = Archivista::BL::Document->new($self,$documentId);
    $self->_documentCollection($documentId,$document);
    $self->curDocId($documentId);
  } else {
    # Getter: check if there is a current selected document
    my $curDocId = $self->curDocId();
    if (defined $curDocId) {
			# There is a document, return the object
			$document = $self->_documentCollection($curDocId);
    } else {
			# There isn't a document, create a new one
			$document = Archivista::BL::Document->new($self);
			$documentId = $document->id;
			$self->_documentCollection($documentId,$document);
    }
  }
  $self->_exception();
  return $document;
}

sub user {
  my $self = shift;
  my $user;
  my $nrOfParam = $#_;
  if ($nrOfParam < 0) {
    # No params -> get the user from the collection
    my $curUserId = $self->curUserId();
    $user = $self->_userCollection($curUserId);
  } elsif ($nrOfParam == 0 or $nrOfParam == 1) {
    my $userId;
    if ($nrOfParam == 0) {
			$userId = shift @_;
    } else {	
			# Two params -> host and username, retrieve the user id
			$user = Archivista::BL::User->new($self);
			$userId = $user->idByHostAndUser(@_);
    }
    $user = Archivista::BL::User->new($self,$userId);
    $self->_userCollection($userId,$user);
    $self->curUserId($userId);
  } elsif ($nrOfParam >= 2) {
    # Three params -> create a new user
    $user = Archivista::BL::User->new($self,undef,@_);
    $self->_userCollection($user->id,$user);
  }
  $self->_exception();
  return $user;
}

sub parameter_with_type {
  my ($self,$parameterId,$type) = @_;
  my $parameter;
  # Check if setter or getter method
  if (defined $parameterId) {
    # Setter: load the requested user
    # and add the object to the collection
    $parameter = Archivista::BL::Parameter->new($self,$type,$parameterId);
    $self->_parameterCollection($parameterId,$parameter);
    $self->curParameterId($parameterId);
  } else {
    # Getter: check if there is a current parameter
    my $curParameterId = $self->curParameterId();
    if (defined $curParameterId) {
			# There is a parameter, return the object
			$parameter = $self->_parameterCollection($curParameterId);
    } else {
			# There isn't a parameter, create a new one
			$parameter = Archivista::BL::Parameter->new($self,$type);
			$parameterId = $parameter->id;
			$self->_parameterCollection($parameterId,$parameter);
    }
  }
  $self->_exception();
  return $parameter;
}

sub box_parameter {
	my ($self,$parameterId) = @_;
 	return $self->parameter_with_type($parameterId,'Archivistabox');
}

sub mask_parameter {
	my ($self,$parameterId) = @_;
	return $self->parameter_with_type($parameterId,$self->mask_name);
}

sub mask_name {
	my ($self,$maskname) = @_;
	if (defined $maskname) { $self->{'mask_name'}=$maskname; }
	return $self->{'mask_name'};
}

sub parameter {
	my ($self,$parameterId) = @_;
	return $self->parameter_with_type($parameterId,'parameter');
}

sub application {
	my ($self,$applicationId) = @_;
	my $application = $self->{'application'};
	$application->id($applicationId) if (defined $applicationId);
  $application->lang($self->lang);
	return $application;
}

sub users {
	my ($self) = @_;
	return Archivista::BL::User->all($self);
}

sub archives {
	my ($self) = @_;
	return Archivista::DL::Archive->all($self->db);
}

sub makeSelectDataFromParamList {
	my ($self,$attr_name,$retDS) = @_;
	$retDS="ARRAY" if !defined $retDS;
  my @idxlst;
	my $idx=0;
  my $anzahl=scalar @{$self->parameter($attr_name)
		->attribute("Inhalt")->barcodes("ARRAY")};
	$anzahl--; # if we have a ocr definition, one minus
  if ($attr_name ne "OCRSets") {
		# add a none value if we have a barcode
    push @idxlst,-1;
    $idx++;
		$anzahl++;
	}
  while ($idx<=$anzahl) {
    push @idxlst,$idx;
    $idx++;
  }
	if ($attr_name eq "OCRSets") {
    push @idxlst,26; # Erfasst
		push @idxlst,27; # Ausschliessen
	}
  if (uc($retDS) eq "ARRAY") {
		return \@idxlst;
  } else {
    my %valueHash;
    my $names=$self->parameter($attr_name)->attribute("Inhalt")
			->barcodes("ARRAY");
		my $counter=0;
		if ($attr_name ne "OCRSets") {
			$valueHash{-1}=$self->lang->string('NO2');
			$counter++;
		}
    for my $nm (@$names) {
     	$valueHash{$counter}=$nm;
      $counter++;
    }
		if ($attr_name eq "OCRSets") {
    	$valueHash{26}=$self->lang->string('OCR_DONE');
			$valueHash{27}=$self->lang->string('OCR_EXCLUDE');
		}
   	return \%valueHash;
  }
}

sub barcodeSelectData {
  my ($self,$retDS) = @_;
  return $self->makeSelectDataFromParamList("Barcodes",$retDS);
}

sub ocrSelectData {
  my ($self,$retDS) = @_;
  return $self->makeSelectDataFromParamList("OCRSets",$retDS);
}

sub scanDefinitions {
  my ($self,$retDS) = @_;
  # POST: pointer to Hash{$defName} -> object of Archivsta::BL::Scan
  #                  Hash{$defName}{'delete'} -> 0/1
  #                  Hash{$defName}{'update'} -> 1
  #       OR
  # 			pointer to Array(DefName)
  # PLEASE NOTE: with my $scanDef = Hash{$defName} we can retrieve the
  # information about the scan definition, for example: $dpi = $scanDef->dpi;
  return $self->parameter("ScannenDefinitionen")
		->attribute("Inhalt")->scans($retDS);
}

sub ocrDefinitions {
  my ($self,$retDS) = @_;
  # POST: pointer to Hash{$defName} -> object of Archivsta::BL::OCR
  #                  Hash{$defName}{'delete'} -> 0/1
  #                  Hash{$defName}{'update'} -> 1
  #       OR
  # 			pointer to Array(DefName)
  # PLEASE NOTE: with my $ocrDef = Hash{$defName} we can retrieve the
  # info about ocr def, i.e: $checkOrientation = $ocrDef->checkOrientation;
  return $self->parameter("OCRSets")->attribute("Inhalt")->ocrs($retDS);
}

sub ocrs {
  my ($self,$retDS) = @_;
  $retDS = "ARRAY" if !defined;
  if (uc($retDS) eq "ARRAY") {
    return $self->ocrDefinitions("ARRAY");
  } else {
    my $phocrdefs = $self->ocrDefinitions("HASH");
    my %lbvalues;
    $lbvalues{-1}="NO";
    while (my($key,$value)=each(%$phocrdefs)) {
			$lbvalues{$key}=$value->name;
    }
    return \%lbvalues;
  }
}

sub ocrlangs {
  my ($self,$retDS) = @_;
  my $nrOfSupportedLanguages = 66;
  my (@ocrids, %ocrhash);
  $retDS = "HASH" if !defined;
  for (my $idx = 0; $idx < $nrOfSupportedLanguages; $idx++) {
    my $id = sprintf "%03d", $idx;
    my $ocrlang = $self->lang->string('OCRLANG_'.$id);
    push @ocrids, $idx;
    $ocrhash{$idx} = $ocrlang;
  }
  # ms - set undef as default (first) value in the selection box 
  unshift @ocrids,-1;
  $ocrhash{-1} = $self->lang->string('NO2');
  if (uc($retDS) eq "ARRAY") {
    return \@ocrids;
  } else {
    return \%ocrhash;
  }
}

sub qualityDefs {
  my ($self,$retDS) = @_;
  if (uc($retDS) eq "ARRAY") {
    return [1,2,3];
  } else {
    my %quality_hash=(1 => $self->lang->string('BEST_QUALITY'), 
                      2 => $self->lang->string('NORMAL_QUALITY'), 
                      3 => $self->lang->string('LOW_QUALITY'));
    return \%quality_hash;
  }
}

sub barcodeDefinitions {
  my ($self,$retDS) = @_; # The same as for scanDefinitions
  return $self->parameter("Barcodes")->attribute("Inhalt")->barcodes($retDS);
}

sub barcodePosition {
  my ($self,$retDS) = @_;
  $retDS = "ARRAY" if (! defined $retDS);
  if (uc($retDS) eq "ARRAY") {
    my @barcodePosition = (0,1);
    return \@barcodePosition;
  } elsif (uc($retDS) eq "HASH") {
    my %barcodePosition;
    $barcodePosition{0} = $self->lang->string("AUTOMATIC");
    $barcodePosition{1} = $self->lang->string("MANUAL_WITH_SETTINGS");
    return \%barcodePosition;
  }
}

sub barcodeTypes {
  my ($self,$retDS) = @_;
  $retDS = "ARRAY" if (! defined $retDS);
  if (uc($retDS) eq "ARRAY") {
    my @barcodeTypes = (1,2,3,4,5,6,7,8,9);
    return \@barcodeTypes;
  } elsif (uc($retDS) eq "HASH") {
    my %barcodeTypes;
    $barcodeTypes{1} = "Code39";
    $barcodeTypes{2} = "Code39 (Check)";
    $barcodeTypes{3} = "Code25";
    $barcodeTypes{4} = "Code25 (Check)";
    $barcodeTypes{5} = "EAN13";
    $barcodeTypes{6} = "Code128";
    $barcodeTypes{7} = "EAN8";
    $barcodeTypes{8} = "ESR";
    $barcodeTypes{9} = "QRCode";
    return \%barcodeTypes;
  }
}

sub barcodeOrientation {
  my ($self,$retDS) = @_;
  $retDS = "ARRAY" if (! defined $retDS);
  if (uc($retDS) eq "ARRAY") {
    my @barcodeOrientation = (1,2,3,4,5);
    return \@barcodeOrientation;
  } elsif (uc($retDS) eq "HASH") {
    my %barcodeOrientation;
    $barcodeOrientation{1} = $self->lang->string("LEFT_TO_RIGHT");
    $barcodeOrientation{2} = $self->lang->string("BOTTOM_TO_TOP");
    $barcodeOrientation{3} = $self->lang->string("RIGHT_TO_LEFT");
    $barcodeOrientation{4} = $self->lang->string("TOP_TO_BOTTOM");
    $barcodeOrientation{5} = $self->lang->string("LEFT_TO_RIGHT_DOWN");
    return \%barcodeOrientation;
  }
}

sub barcodeVerticalStretch {
  my ($self,$retDS) = @_;
  $retDS = "ARRAY" if (! defined $retDS);
  if (uc($retDS) eq "ARRAY") {
    my @barcodeVerticalStretch = (0,1,2);
    return \@barcodeVerticalStretch;
  } elsif (uc($retDS) eq "HASH") {
    my %barcodeVerticalStretch;
    $barcodeVerticalStretch{0} = $self->lang->string("NO_CORRECTION");
    $barcodeVerticalStretch{1} = $self->lang->string("WITH_DOUBLE_HEIGHT");
    $barcodeVerticalStretch{2} = $self->lang->string("WITH_TRIPLE_HEIGHT");
    return \%barcodeVerticalStretch;
  }
}

sub barcodeProcessNumber {
  my ($self) = @_;
	my @barcodeProcessNumber = ();
	for (my $c=1;$c<=48;$c++) {
	  push @barcodeProcessNumber,$c;
	}
  return \@barcodeProcessNumber;
}

sub barcodeProcessLength {
  my ($self,$retDS) = @_;
  $retDS = "ARRAY" if (! defined $retDS);
  if (uc($retDS) eq "ARRAY") {
    my @barcodeProcessLength;
    for (my $idx = 0; $idx <= 127; $idx++) {
			push @barcodeProcessLength, $idx;
    }
    return \@barcodeProcessLength;
  } elsif (uc($retDS) eq "HASH") {
    my %barcodeProcessLength;
    $barcodeProcessLength{0} = $self->lang->string("ALL");
    for (my $idx = 1; $idx <= 128; $idx++) {
			$barcodeProcessLength{$idx} = $idx;
    }
    return \%barcodeProcessLength;
  }
}

sub barcodeProcessAttributes {
  my ($self,$retDS) = @_;
  my $idx = 2;
  $retDS = "ARRAY" if (! defined $retDS);
  if (uc($retDS) eq "ARRAY") {
    my $paattributes = $self->db->userDefinedAttributes;
		# +1 is for DOKNR (Akte/Document) -> comes from hash
    my @attributes = (0 .. $#$paattributes + $idx + 1);
    return \@attributes;
  } elsif (uc($retDS) eq "HASH") {
    my %attributes;
    my $paattributes = $self->db->userDefinedAttributes;
    $attributes{0} = $self->lang->string("TITLE");
    $attributes{1} = $self->lang->string("DATE");
    foreach my $attribute (@$paattributes) {
    	$attributes{$idx} = $attribute;
	    $idx++;
    }
		# ducment is also available (docnr=barcode -> does already exists)
		$attributes{$idx} = $self->lang->string("DOKNR");
    return \%attributes;
  }
}

sub barcodeProcessStart {
  my ($self,$retDS) = @_;
  my @barcodeProcessStart;
  for (my $idx = 1 ; $idx <= 128; $idx++) {
    push @barcodeProcessStart, $idx;
  }
  return \@barcodeProcessStart;
}

sub barcodeProcessCharacter {
  my ($self,$retDS) = @_;
  $retDS = "ARRAY" if (! defined $retDS);
  if (uc($retDS) eq "ARRAY") {
    my @barcodeProcessCharacter;
    for (my $idx = 0; $idx <= 127; $idx++) {
			push @barcodeProcessCharacter, $idx;
    }
    return \@barcodeProcessCharacter;
  } elsif (uc($retDS) eq "HASH") {
    my %barcodeProcessCharacter;
    $barcodeProcessCharacter{0} = $self->lang->string("ALL");
    for (my $idx = 1; $idx <= 128; $idx++) {
			$barcodeProcessCharacter{$idx} = $idx;
    }
    return \%barcodeProcessCharacter;
  }
}

sub barcodeNumberOf {
  my ($self) = @_;
  return 48;
}

sub barcodeProcessNumberOf {
	my ($self) = @_;
  return 32;
}

sub userLevelsByArray {
	my ($self) = @_;
	my @userLevels = (0,1,2,3,250,255);
	return \@userLevels;
}

sub userLevelsByHash {
	my ($self) = @_;
	my %userLevel;
  $userLevel{0} = $self->lang->string("LEVEL_0");
  $userLevel{1} = $self->lang->string("LEVEL_1");
  $userLevel{2} = $self->lang->string("LEVEL_2");
  $userLevel{3} = $self->lang->string("LEVEL_3");
  $userLevel{250} = $self->lang->string("LEVEL_250");
  $userLevel{255} = $self->lang->string("LEVEL_255");
  return \%userLevel;
}

sub attributeDataTypesByArray {
	my ($self) = @_;
	# This values must agree with Archivista::DL::Archive::attributeDbType!
	my @attributeDataTypes = ("varchar","int","double","datetime","tinyint");
	return \@attributeDataTypes;
}

sub attributeDataTypesByHash {
	my ($self) = @_;
	# Hash keys are the same values as in sub attributeDataTypesByArray
	my %attributeDataTypes;
  $attributeDataTypes{'varchar'} = $self->lang->string("TEXT");
  $attributeDataTypes{'int'} = $self->lang->string("NUMBER_INTEGER");
  $attributeDataTypes{'double'} = $self->lang->string("NUMBER_DOUBLE");
  $attributeDataTypes{'datetime'} = $self->lang->string("DATETIME");
  $attributeDataTypes{'tinyint'} = $self->lang->string("YES_NO");
  return \%attributeDataTypes;
}

sub passwordTypes {
	my ($self,$retDS) = @_; # Return datastructure (ARRAY or HASH)
	$retDS = "ARRAY" if (! defined $retDS);
	if (uc($retDS) eq "ARRAY") {
  	my @pwTypes = (0,1,2,3);
		return \@pwTypes;
	} elsif (uc($retDS) eq "HASH") {
  	my %pwTypes;
    $pwTypes{'0'} = $self->lang->string("NO_PASSWORD");
    $pwTypes{'1'} = $self->lang->string("NORMAL_LOGIN");
    $pwTypes{'2'} = $self->lang->string("NEW_PASSWORD_EMPTY_NOT_ALLOWED");
    $pwTypes{'3'} = $self->lang->string("NEW_PASSWORD_EMPTY_ALLOWED");
    return \%pwTypes;
  }
}

sub imageTypes {
	my ($self,$retDS) = @_;
	$retDS = "ARRAY" if (! defined $retDS);
	if (uc($retDS) eq "ARRAY") {
		my @imageTypes = (1,2,3);
		return \@imageTypes;
	} elsif (uc($retDS) eq "HASH") {
  	my %imageTypes;
    $imageTypes{'1'} = $self->lang->string("TIFF");
    $imageTypes{'2'} = $self->lang->string("PNG");
    $imageTypes{'3'} = $self->lang->string("JPEG");
    return \%imageTypes;
	}
}

sub scanTypes {
	my ($self,$retDS) = @_;
	$retDS = "ARRAY" if (! defined $retDS);
	if (uc($retDS) eq "ARRAY") {
		my @scanTypes = (0,1,2);
		return \@scanTypes;
	} elsif (uc($retDS) eq "HASH") {
		my %scanTypes;
		$scanTypes{'0'} = $self->lang->string("BLACK_WHITE");
		$scanTypes{'1'} = $self->lang->string("GRAYSCALES_8_BIT");
		$scanTypes{'2'} = $self->lang->string("COLOR_24_BIT");
		return \%scanTypes;
	}
}

sub scanRotations {
	my ($self,$retDS) = @_;
	$retDS = "ARRAY" if (! defined $retDS);
	if (uc($retDS) eq "ARRAY") {
		my @scanRotations = (0,1,2,3);
		return \@scanRotations;
	} elsif (uc($retDS) eq "HASH") {
		my %scanRotations;
		$scanRotations{'0'} = "0";
		$scanRotations{'1'} = "90";
		$scanRotations{'2'} = "180";
		$scanRotations{'3'} = "270";
		return \%scanRotations;
	}
}

sub adfTypes {
	my ($self,$retDS) = @_;
	$retDS = "ARRAY" if (! defined $retDS);
	if (uc($retDS) eq "ARRAY") {
		my @adfTypes = (1,-1,-2,-3);
		return \@adfTypes;
	} elsif (uc($retDS) eq "HASH") {
		my %adfTypes;
		$adfTypes{'1'} = "ADF simplex";
		$adfTypes{'-1'} = "ADF duplex";
		$adfTypes{'-2'} = "ADF 270/90";
		$adfTypes{'-3'} = "ADF Rear";
		return \%adfTypes;
	}
}

sub lang {
	my ($self,$langCode) = @_;
	if (defined $langCode) {
		$self->{'lang_code'} = $langCode;
		$self->{'languages'} = Archivista::BL::Languages->new($self,$langCode);
	} elsif (defined $self->{'lang_code'}) {
		$self->{'languages'} =
			Archivista::BL::Languages->new($self,$self->{'lang_code'});
	} else {
		$self->{'lang_code'} = "de";
		$self->{'languages'} = 
			Archivista::BL::Languages->new($self,$self->{'lang_code'});
	}
	return $self->{'languages'};
}

sub documentOwners {
	my ($self,$retDS) = @_; # Return data structure (ARRAY, HASH) array default
	$retDS = "ARRAY" if (! defined $retDS);
	$retDS = uc($retDS);
	my $pdocOwners = Archivista::DL::AvUser->documentOwners($self->db,$retDS);
	if ($retDS eq "ARRAY") {
		unshift @$pdocOwners, "";
	} elsif ($retDS eq "HASH") {
		$$pdocOwners{''} = $self->lang->string("ALL");
	}
	return $pdocOwners;
}

sub maskDefinitions {
	my ($self) = @_;
	# POST: pointer to Hash(maskDefinitionNumber,maskDefinitionName)
	my $p1 = Archivista::DL::Parameter->maskDefinitions($self->db);
	return $p1;
}

sub maskNextDefinition {
	my ($self) = @_;
	my $phdefinitions = Archivista::DL::Parameter->maskDefinitions($self->db);
	my @definitions = sort keys %$phdefinitions;
	my $maxDefinition = pop @definitions;
	my $maxIntDefinition = int $maxDefinition;
	my $nextIntDefinition = $maxIntDefinition + 1;
	my $nextDefinition = sprintf "%02d", $nextIntDefinition;
	return $nextDefinition;
}

sub maskFieldTypes {
	my ($self) = @_;
	my %maskFieldTypes;
  # Please note: the keys for the hash must be the same as saved on FelderObj at
  # the fifth position (x;y;w;h;type;)
  $maskFieldTypes{'0'} = $self->lang->string("NORMAL");
  $maskFieldTypes{'4'} = $self->lang->string("DEFINITION");
  $maskFieldTypes{'5'} = $self->lang->string("1TON");
  $maskFieldTypes{'6'} = $self->lang->string("NUMBER_CODE");
  $maskFieldTypes{'7'} = $self->lang->string("MULTI");
  $maskFieldTypes{'3'} = $self->lang->string("TEXT_CODE");
  $maskFieldTypes{'8'} = $self->lang->string("FASTSORT");
	return \%maskFieldTypes;	
}

sub maskDefinition {
  my ($self,$definition,$retDS) = @_;
	# PRE: mask definition number (i.e. 00, 01 ...)
	# POST: pointer to Hash{$fieldName}{'field_name'}
	# 								 Hash{$fieldName}{'fieldObj object'}
	# 								 Hash{$fieldName}{'fieldTab object'}
	$retDS = "HASH" if (! defined $retDS);
	if (uc($retDS) eq "HASH") { 
		my %maskDefinition;
		# Retrieve data (FelderObj/FelderTab) for all fields of a definition 
		my $phfieldsObj = $self->mask_parameter("FelderObj".$definition)
			->attribute("Inhalt")->fields;
		my $phfieldsTab = $self->mask_parameter("FelderTab".$definition)
			->attribute("Inhalt")->fields("HASH");
		foreach my $fieldName (keys %$phfieldsObj) {
			$maskDefinition{$fieldName}{'field_name'} = $fieldName;
			$maskDefinition{$fieldName}{'field_object'} = $$phfieldsObj{$fieldName};
			$maskDefinition{$fieldName}{'field_tab'} = $$phfieldsTab{$fieldName};
		}
		return \%maskDefinition;
	} else {
		return $self->mask_parameter("FelderTab".$definition)
			->attribute("Inhalt")->fields("ARRAY");
	}
}

sub userSqlDefinitions {
	my ($self) = @_;
	return Archivista::DL::Parameter->userSqlDefinitions($self->db);
}

sub session {
	my ($self) = @_;
	return $self->{'session'};
}

sub jobs {
	my ($self,$job) = @_;
	$self->{'jobs'} = Archivista::Util::Jobs->new($self->db);
	$self->{'jobs'}->param("job",$job);
  return $self->{'jobs'};
}

sub grant {
	my ($self,$pausers) = @_;	# 2-dim array[n][3] host,user,level
	Archivista::BL::User->grant($self,$pausers);
}

sub revoke {
	my ($self,$pausers) = @_;	# 2-dim array[n][3] host,user,level
	Archivista::BL::User->revoke($self,$pausers);
}

sub clearDocument {
	my ($self,$documentId) = @_;
	undef $self->{'cur_document_id'};
	if (defined $documentId) {
		$self->collection("document")->remove($documentId);
	} else {
		$self->collection("document")->clear;
	}
}

sub clearUser {
	my ($self,$userId) = @_;
	undef $self->{'cur_user_id'};
	if (defined $userId) {
		$self->collection("user")->remove($userId);
	} else {
		$self->collection("user")->clear;
	}
}

sub clearParameter {
	my ($self,$parameterId) = @_;
	undef $self->{'cur_parameter_id'};
	if (defined $parameterId) {
		$self->collection("parameter")->remove($parameterId);
	} else {
		$self->collection("parameter")->clear;
	}
}

sub hostIsSlave {
	my ($self) = @_;
	return $self->db->hostIsSlave();
}

sub doublefeedTypes {
	my ($self,$retDS) = @_;
	$retDS = "ARRAY" if (! defined $retDS);
	if (uc($retDS) eq "ARRAY") {
		my @ddTypes = (0,1,2,3,4);
		return \@ddTypes;
	} elsif (uc($retDS) eq "HASH") {
		my %ddTypes;
		$ddTypes{'0'} = $self->lang->string("SCAN_DOUBLEFEED_NONE");
		$ddTypes{'1'} = $self->lang->string("SCAN_DOUBLEFEED_DEFAULT");
		$ddTypes{'2'} = $self->lang->string("SCAN_DOUBLEFEED_THICKNESS");
		$ddTypes{'3'} = $self->lang->string("SCAN_DOUBLEFEED_LENGTH");
		$ddTypes{'4'} = $self->lang->string("SCAN_DOUBLEFEED_BOTH");
		return \%ddTypes;
	}
}

1;

